"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reactFrameworkPlugin = void 0;
const vfs_1 = require("@previewjs/vfs");
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const extract_component_1 = require("./extract-component");
const optimize_deps_plugin_1 = require("./optimize-deps-plugin");
const react_imports_plugin_1 = require("./react-imports-plugin");
const special_types_1 = require("./special-types");
/** @deprecated */
exports.reactFrameworkPlugin = {
    isCompatible: async (dependencies) => {
        var _a;
        const version = await ((_a = dependencies["react"]) === null || _a === void 0 ? void 0 : _a.readInstalledVersion());
        if (!version) {
            return false;
        }
        return parseInt(version) >= 16;
    },
    async create() {
        const previewDirPath = path_1.default.resolve(__dirname, "..", "preview");
        return {
            pluginApiVersion: 3,
            name: "@previewjs/plugin-react",
            defaultWrapperPath: "__previewjs__/Wrapper.tsx",
            previewDirPath,
            specialTypes: special_types_1.REACT_SPECIAL_TYPES,
            tsCompilerOptions: {
                jsx: typescript_1.default.JsxEmit.ReactJSX,
                jsxImportSource: "react",
            },
            transformReader: (reader, rootDirPath) => (0, vfs_1.createStackedReader)([
                reader,
                (0, vfs_1.createFileSystemReader)({
                    mapping: {
                        from: path_1.default.join(previewDirPath, "types"),
                        to: path_1.default.join(rootDirPath, "node_modules", "@types"),
                    },
                    watch: false,
                }),
            ]),
            detectComponents: async (typeAnalyzer, absoluteFilePaths) => {
                const resolver = typeAnalyzer.analyze(absoluteFilePaths);
                const components = [];
                for (const absoluteFilePath of absoluteFilePaths) {
                    components.push(...(0, extract_component_1.extractReactComponents)(resolver, absoluteFilePath));
                }
                return components;
            },
            viteConfig: () => {
                return {
                    resolve: {
                        alias: {
                            "react-native": "react-native-web",
                        },
                    },
                    plugins: [(0, optimize_deps_plugin_1.optimizeReactDepsPlugin)(), (0, react_imports_plugin_1.reactImportsPlugin)()],
                    define: {
                        "process.env.RUNNING_INSIDE_PREVIEWJS": "1",
                    },
                };
            },
        };
    },
};
exports.default = exports.reactFrameworkPlugin;
